import os
import sys
from logging import getLogger

from opentelemetry import trace
from opentelemetry.sdk.resources import Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchExportSpanProcessor
from pkg_resources import iter_entry_points

from splunk_otel.exporter.zipkin import ZipkinSpanExporter

logger = getLogger(__file__)

DEFAULT_SERVICE_NAME = "unnamed-python-service"
DEFAULT_ENDPOINT = "http://localhost:9080/v1/trace"


def start_tracing(url=None, service_name=None):
    init_tracer(url, service_name)
    auto_instrument()


def init_tracer(url=None, service_name=None):
    if not url:
        url = os.environ.get(
            "OTEL_EXPORTER_ZIPKIN_ENDPOINT",
            DEFAULT_ENDPOINT,
        )

    if not service_name:
        service_name = os.environ.get(
            "SPLK_SERVICE_NAME",
            DEFAULT_SERVICE_NAME,
        )

    provider = TracerProvider(
        resource=Resource.create(
            attributes={
                "service.name": service_name,
            }
        )
    )
    trace.set_tracer_provider(provider)

    exporter = ZipkinSpanExporter(url=url, service_name=service_name)
    provider.add_span_processor(BatchExportSpanProcessor(exporter))


def auto_instrument():
    for entry_point in iter_entry_points("opentelemetry_instrumentor"):
        try:
            entry_point.load()().instrument()  # type: ignore
            logger.debug(
                "Instrumented %s",
                entry_point.name,
            )

        except Exception:  # pylint: disable=broad-except
            logger.exception(
                "Instrumenting of %s failed",
                entry_point.name,
            )



import os
from typing import Sequence

from opentelemetry.exporter.zipkin import ZipkinSpanExporter as BaseZipkinSpanExporter
from opentelemetry.trace import Span


class ZipkinSpanExporter(BaseZipkinSpanExporter):
    def __init__(self, *args, **kwargs):
        self._max_attribute_length = int(os.environ.get("SPLK_MAX_ATTR_LENGTH", 0))
        super().__init__(*args, **kwargs)

    def _translate_to_zipkin(self, spans: Sequence[Span]):
        zipkin_spans = super()._translate_to_zipkin(spans)
        if not self._max_attribute_length:
            return zipkin_spans

        for span in zipkin_spans:
            tags = span.get("tags", {})
            for key, val in tags.items():
                if len(val) > self._max_attribute_length:
                    tags[key] = val[: self._max_attribute_length]
        return zipkin_spans



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['splunk_otel', 'splunk_otel.cmd', 'splunk_otel.exporter', 'splunk_otel.site']

package_data = \
{'': ['*']}

install_requires = \
['opentelemetry-api>=0.13b0,<0.14',
 'opentelemetry-exporter-zipkin>=0.13b0,<0.14',
 'opentelemetry-instrumentation>=0.13b0,<0.14',
 'opentelemetry-sdk>=0.13b0,<0.14']

entry_points = \
{'console_scripts': ['splk-py-trace = splunk_otel.cmd.trace:run',
                     'splk-py-trace-bootstrap = splunk_otel.cmd.bootstrap:run']}

setup_kwargs = {
    'name': 'splunk-opentelemetry',
    'version': '0.1.3',
    'description': 'The Splunk distribution of OpenTelemetry Python Instrumentation provides a Python agent that automatically instruments your Python application to capture and report distributed traces to SignalFx APM.',
    'long_description': None,
    'author': 'Splunk',
    'author_email': 'splunk-oss@splunk.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
